#' @importFrom GSA GSA

ConductGSATest <- function(alpha = 0.005,
                           gsaSurv=FALSE,
                           gsaPaired=FALSE,
                           gsaQuant=FALSE,
                           gsanperm=1000,
                           mingenes=1,
                           maxgenes=5000,
                           filepath) {

 # source(paste(ArrayToolsPath,"/R/BiocUtil.R", sep=""), echo = FALSE)
 # print("Loading R package: GSA ...")
 # if (!LoadRPackage("GSA", isBioconductor=F)) return(FALSE)
 # if (!LoadRPackage("Biobase", isBioconductor=T)) return(FALSE)

  #read exprSet object
  print("Loading data and gene sets ...")
  load(file=paste(filepath, "/exprSet.rda", sep=""))
  if (gsaQuant) load(file=paste(filepath, "/experiments.rda", sep=""))
  load(file=paste(filepath, "/GeneSetList.rda", sep=""))
  GeneSets <- strsplit(GeneSetList, " ")

  #conduct GSA test
  print(paste("Conducting GSA test for", length(GeneSets), "gene sets ..."))
  censorvar <- NULL
  outcome.type <- "Two class unpaired"
  if (gsaPaired) {
    outcome.type = "Two class paired"
    #Note: for GSA, paired data format is (-1,1,-2,2  etc)
    #where -1,1 represent a pair (eg before and after)
    #data can be in any order  eg (2,-1,-2,1)  is ok
    #
    #Note also the expression data matrix in exprSet has been sorted
    # by the pheno class and pair ID, see WriteDataToFiles.R
    # y can be defined as:
    npairs <- length(pData(aExprSet)$CLASS)/2
    y <- c(-1*rep(1:npairs),rep(1:npairs))
  }
  if (gsaSurv) {
    outcome.type = "Survival"
    y <- as.numeric(pData(aExprSet)$TIMEsurvival)
    censorvar <- as.numeric(pData(aExprSet)$EVENTdeath)
  }
  if (gsaQuant){
    outcome.type = "Quantitative"
    y <- experiments$qt
  }

  if (outcome.type == "Two class unpaired") {
    nlevel <- length(levels(pData(aExprSet)$CLASS))
    if (nlevel > 2) outcome.type = "Multiclass"
    y <- as.numeric(pData(aExprSet)$CLASS)
  }
  x <- exprs(aExprSet)
  genenames <- featureNames(aExprSet)

  out <- try(GSA.obj <- GSA(x,y, genesets=GeneSets, genenames=genenames, method="maxmean",
                            resp.type=outcome.type, censoring.status=censorvar, random.seed=100,
                            minsize=mingenes, maxsize=maxgenes, nperms=gsanperm))
  if (class(out)=='try-error') {
    #an error identified when using GSA function for a collection of gene sets (BioCarta pathways).
    #The error message is Error in var(sds) : missing observations in cov/cor
    #from the est.e0() function (called by GSA.func(), in turn by GSA()), in the line cv.sd[j] <- sqrt(var(sds))/mean(sds)
    #The error occurs seemly depending on the collection of gene sets used.
    #It is found it always occurs when the UNIQUE number of matched genes from all testing gene sets is less than 100.
    #Then, in the est.e0() function, when the above condition happens, the sds consists of zero and NA.
    #therefore the line cv.sd[j] <- sqrt(var(sds))/mean(sds) fails!
    #This error can be reproduced by filtering the project to a few hundreds of genes than run gene set comparison.
    #Sent email to Dr. Tibshirani on this 1/24/2008, no reply.
    #Current solution is to add try() to handle such error. 2/6/2008
    cat(geterrmessage(), file=stderr())
    return (NULL)
  }

  #1/9/08 skip using GSA.listsets since it won't output p-values for negative gene sets when outcome.type is Multiclass.
  #GSA.list <- GSA.listsets(GSA.obj, geneset.names=names(GeneSets), FDRcut=2)
  #all.list <- rbind(GSA.list$negative, GSA.list$positive)
  #all.list.sort <- all.list[order(as.numeric(all.list[,"Gene_set"])),] #restore to the order of input genesets
  #pValue <- as.numeric(all.list.sort[,"p-value"])
  #pFDR <- as.numeric(all.list.sort[,"FDR"])
  #GSAscore <- as.numeric(all.list.sort[,"Score"])

  GeneSetIndex <- 1:length(GeneSets)
  nGenes <- GSA.obj$ngenes
  GSAscore <- GSA.obj$GSA.scores
  pValue <- GSA.obj$pvalues.lo; #initialize
  pValue[which(GSAscore<0)] <- GSA.obj$pvalues.lo[which(GSAscore<0)]
  pValue[which(GSAscore>=0)] <- GSA.obj$pvalues.hi[which(GSAscore>=0)]

  return (data.frame(Index=GeneSetIndex, nGenesInGeneSet=nGenes, pValue=pValue, GSAscore=GSAscore))
}
