CreateGeneListDataFile <- function(geneId,
                                   filter,
                                   UserGeneListFileNames,
                                   GeneSetMin,
                                   GeneSetMax,
                                   Organism,
                                   WorkPath) {
  #FtrMaxLength is the maximum number of genes Fortran can handle in GOComparison.exe.
  FtrMaxLength <- 5000
  # ReturnList <- list(Msg="", MaxLength=0)
  #
  # GeneIdFileName <- "GeneIdsColumn.rda"
  # GeneIdFullFileName <- paste(WorkPath,"/",GeneIdFileName,sep="")
  # load(GeneIdFullFileName)
  # GeneIDs <- as.vector(as.character(GeneIdsColumn))
  #
  # #added probe reduction consideration. 1/21/2008
  # load(paste(ProjectPath, "BinaryData/ApplyFilter.rda", sep="/"))
  # # modify for the change of moving the probe reduction to the main dialog in v3.9, MLI 1/11/2010
  # #load(paste(WorkPath, "ProbeReductionIndex.rda", sep="/"))
  # load(paste(WorkPath, "GeneId1.rda", sep="/"))
  # ProbeReductionIndex <- as.numeric(names(GeneId1[,1])) #the index is row number of the full set of genes
  #
  # NewFilter <- c(1:length(Result$Filter))[Result$Filter==1]
  # GeneIDs <- GeneIDs[NewFilter %in% ProbeReductionIndex]
  #
  # nspot <- length(GeneIDs)
  # load(paste(WorkPath,"/GeneListFileNames.rda",sep=""))
  # UserGeneListFileNames <- GeneListFileNames
  # NUserLists <- length(UserGeneListFileNames)
  # NLists <- NUserLists
  # GeneList <- rep("", NLists)
  # GeneListLength <- rep(0, NLists)
  # MaxLength <- 0


  nspot <- sum(filter)
  GeneIds <- geneId[filter == 1,]
  NUserLists <- length(UserGeneListFileNames)
  NLists <- NUserLists
  GeneList <- rep("", NLists)
  GeneListLength <- rep(0, NLists)
  MaxLength <- 0

  OutPutFileName <- "GeneSet_data.txt"
  SelectedColumn <- "Symbol"# # fixed for the TF and microRNA case
  if (tolower(SelectedColumn) == "symbol" || tolower(SelectedColumn) == "gene.symbol") { SelectedColumn <- "Symbol" }
  if (tolower(SelectedColumn) == "clone" || tolower(SelectedColumn) == "cloneid") { SelectedColumn <- "CloneID" }
  if (tolower(SelectedColumn) == "ugcluster" || tolower(SelectedColumn) == "ug.cluster") { SelectedColumn <- "UGCluster" }
  if (tolower(SelectedColumn) == "gbacc" || tolower(SelectedColumn) == "gb.acc"|| tolower(SelectedColumn)=="accession") { SelectedColumn <- "Accession" }  #Lori 08/31/10

  GeneSets <- list()
  for(i in 1:NLists) {

    #Tempxx <- read.table(UserGeneListFileNames[i], header=T, sep="\t", as.is = TRUE, na.strings = "NA")
    #8/28/07 Yong modified to still use read.table to get headername (consistent with GeneIds name),
    #but use scan to read data because read.table can not handle speical character single quote ', it will break code.
    try.out<-try(HeaderRow <- names(read.table(UserGeneListFileNames[i], sep="\t", header=T, nrows=1)))
    if(class(try.out)=='try-error') { next } #Qian 2/17/2010 skip sub-folders.
    RowsIn <- scan(UserGeneListFileNames[i], sep="\n", what="", skip=1, quiet=T)
    #Qihao modified codes to skip file that does not have gene and give an warning to user. 8/15/2011
    if(length(RowsIn)<1){
     # library(tcltk)
     # tkmessageBox(message=paste('There are not genes in the genelist file ',UserGeneListFileNames[i], '.\nThe GSEA R package will skip it and proceed to the other.',sep=''),title='Warning',icon='warning')
      next
    }
    RawData <- lapply(RowsIn, function(x) {
      if(length(unlist(strsplit(x, "\t")))>=length(HeaderRow)) unlist(strsplit(x, "\t"))[1:length(HeaderRow)]
      else c(unlist(strsplit(x,"\t")), rep("", length(HeaderRow)-length(unlist(strsplit(x,"\t")))))
    })
    RowsMatrix <- matrix(unlist(RawData), byrow=T, nrow=length(RowsIn), ncol=length(HeaderRow))
    colnames(RowsMatrix) <- HeaderRow
    Tempxx <- as.data.frame(RowsMatrix)

    ColPosition <- 0
    header <- toupper(colnames(Tempxx))
    header <- gsub("GENE.SYMBOL", "SYMBOL", header)
    header <- gsub("UG.CLUSTER", "UGCLUSTER", header) #make sure UGCluster is consistent too.
    if (toupper(SelectedColumn) %in% header) {
      ColPosition <- which(header == toupper(SelectedColumn))
    } else { next }

    if (length(ColPosition)==0) { #no matched header found
      break
    }

    GeneIDs <- GeneIds[,toupper(colnames(GeneIds))==toupper(SelectedColumn)]
    GeneListCurrentIDs <- Tempxx[, ColPosition]
    GeneIDs <- toupper(GeneIDs)
    if (length(GeneListCurrentIDs)>1) { GeneListCurrentIDs <- toupper(GeneListCurrentIDs) }
    GenesInGeneList <- (1:nspot)[is.element(GeneIDs,GeneListCurrentIDs) & GeneIDs != ""]
    ListLength <- length(GenesInGeneList)
    if(ListLength < GeneSetMin || ListLength > GeneSetMax) { next } #add control of length of meatched gene set, 9/20/07
    if(ListLength > MaxLength) MaxLength <- ListLength
    if(ListLength > 0) {
      GeneList[i] <- paste(GenesInGeneList,collapse = " ")
      GeneListLength[i] <- ListLength
    }
    GeneSets[[i]] <- GenesInGeneList
  }

  if(MaxLength > 0) {
    if (MaxLength <= FtrMaxLength) {
      GeneSetName <- UserGeneListFileNames[GeneList!=""]
      save(GeneSetName,file=paste(WorkPath,"/GeneSetName.rda",sep=""))

      GeneSetList <- GeneList[GeneList!=""]
      save(GeneSetList, file=paste(WorkPath,"/GeneSetList.rda",sep=""))

      FirstLine <- paste(length(GeneSetName), MaxLength,sep=" ")
      GeneSetLength <- GeneListLength[GeneList!=""]
      GeneList <- c(FirstLine, paste(GeneSetLength, GeneSetList, sep=" "))
      write.table(GeneList, file=paste(WorkPath,"/", OutPutFileName, sep=""), row.names = F,col.names = F, quote = F)
    } else {
      # ReturnList$Msg <- paste("ERROR: The number of genes in one gene set is larger than ",FtrMaxLength, ". The analyis quits now. ",
      #                         "Please subset your genes and rerun the analysis.", sep="")
      # return(ReturnList)
      stop("ERROR: The number of genes in one gene set is larger than ",FtrMaxLength, ". The analyis quits now. ",
           "Please subset your genes and rerun the analysis.")
    }
  }

 # ReturnList$MaxLength <- MaxLength
#  return(ReturnList)
  return(GeneSets)
}

