##############################################################################
#Function for creating gene set specific heatmap
#gsindex: gene set index
#gsorder: gene set order
#singlechannel: T or F
##############################################################################
CreateGeneSetSpecificHeatmap <- function(geneId,
                                         filter,
                                         gsindex,
                                         gsorder,
                                         #surv,
                                         trunclevel, singlechannel,
                                         projectpath, workpath,
                                         outputpath, paired,
                                         DoGroupComparison,
                                         grp) {

  print("Loading data and gene sets ...")
  load(file=paste(workpath, "/exprSet.rda", sep=""))
  load(file=paste(workpath, "/GeneSetList.rda", sep=""))
  load(file=paste(workpath, "/GeneSetName.rda", sep=""))
  #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
  if(paired) {load(file=paste(workpath, "/experiments.rda", sep=""))}
  GeneSets <- strsplit(GeneSetList, " ")
  GeneSets <- GeneSets[gsindex]
  GeneSets <- GeneSets[gsorder]
  GeneSetName <- GeneSetName[gsindex]
  GeneSetName <- GeneSetName[gsorder]
  GeneSetName <- gsub(".txt","",unlist(lapply(GeneSetName, function(x) rev(unlist(strsplit(x,"/")))[1])))

#  require(Biobase)
  dat <- exprs(aExprSet)
  pdat <- pData(aExprSet)
  #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
  if(paired) {pairedid<-experiments$id}

  #sort data by pheno or pairedid
  if(DoGroupComparison) {
    pheno <- paste(grp, pdat$CLASS, sep=":") #Qian 9/25/09
  } else {
    pheno <- pdat$CLASS
  }

  #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
  pheno.sort <- order(pheno)
  pheno <- pheno[pheno.sort]
  dat <- dat[,pheno.sort]
  arrayid <- colnames(dat)

  if(paired) {
    pheno.sort<-order(pheno, pairedid)
    pairedid<-pairedid[pheno.sort]
    pheno <- pheno[pheno.sort]
    dat <- dat[,pheno.sort]
    if(DoGroupComparison) {grp <- grp[pheno.sort]}
    #Half data
    newcolumns = length(pairedid)/2
    pairedid1 = pairedid[1:newcolumns]
    pheno1 = pheno[1:newcolumns]
    dat1 = dat[,1:newcolumns]
    for(i in 1:newcolumns){
      pairedid1[i] = pairedid[newcolumns+i]
      pheno1[i] = pheno[newcolumns+i]
      for(k in 1:dim(dat)[1]) {
        if(dat[k,i] < -999 || dat[k,newcolumns+i] < -999) {
          dat1[k,i] = -9999999
        } else {
          dat1[k,i] = dat[k,i] - dat[k,newcolumns+i]
        }
      }
    }
    arrayid = pairedid1
    pheno = pheno1
    dat = dat1
    if(DoGroupComparison) {pheno <- grp[1:newcolumns]}
  }

  #get gene symbol
#  load(paste(workpath, "GeneId1.rda", sep="/"))
#  PrimaryIDFiltered <- GeneId1[,1] #GeneId1 is a matrix
#  FilteredIndex <- as.numeric(names(PrimaryIDFiltered)) #the index is row number of the full set of genes
  # Ting 7/16/19
  PrimaryIDFiltered <- geneId[,1][filter==1]
  FilteredIndex <- (1:nrow(geneId))[filter == 1]
  # modify for the change of moving the probe reduction to the main dialog in v3.9, MLI 1/11/2010
  #load(paste(workpath, "ProbeReductionIndex.rda", sep="/"))
  #FilteredIndex <- FilteredIndex[FilteredIndex %in% ProbeReductionIndex]

  # Ting 7/16/19
  # ProjectAnnotation <- paste(projectpath, "Annotations", sep="/")
  # GeneSymbolFile <- paste(ProjectAnnotation, "Symbol.txt", sep="/")
  # GeneSymbol <- scan(file=GeneSymbolFile, what="", skip=1, sep="\n", quiet=T)
  # GeneSymbol <- unlist(lapply(strsplit(GeneSymbol, "\t"), function(x) x[2]))
  # GeneSymbol <- unlist(lapply(strsplit(GeneSymbol, " "), function(x) x[1])) # change such as "TNS1 /// AKAP9" to "TNS1"
  # GeneSymbolFiltered <- GeneSymbol[FilteredIndex]

  ind <- grep("symbol",colnames(geneId),ignore.case=TRUE,value=FALSE)
  if (length(ind) == 0){
    stop("The argument geneId must contain a column for gene symbols and the name for that column should contain \"symbol\".\n")
  }
  GeneSymbol <- geneId[,ind]


  GeneSymbol <- unlist(lapply(strsplit(GeneSymbol, " "), function(x) x[1])) # change such as "TNS1 /// AKAP9" to "TNS1"
  GeneSymbolFiltered <- GeneSymbol[FilteredIndex]

  #draw gene set specific heatmap
  dir.create(outputpath, showWarnings=F)
  unlink(paste(outputpath, '/*', sep=''))
#  source(paste(ArrayToolsPath, "R/BRBHeatmap.R", sep="/"))
  for(i in 1:length(GeneSets)){
    tempz <- dat[as.numeric(GeneSets[[i]]),,drop=F]

    #Qian 6/26/09
    BRBHeatmap(AnalysisName = "GenesetComparison",
               pngfilename = paste(outputpath, '/geneset', i , '.png', sep=''),
               xdata = tempz,
               arrayid = arrayid, #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
               classid = pheno,
               geneid = GeneSymbolFiltered[as.numeric(dimnames(tempz)[[1]])],
               ArrayToolsPath = ArrayToolsPath,
               isSingleChannel = singlechannel,
               GenesetName = GeneSetName[i],
               paired = paired, DoGroupComparison = DoGroupComparison) #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
    #EndQian
  }
  GeneSetHeatmapHtml(outputpath, n=length(GeneSets))
}
