#Ting 8/11/2014  added to support predicted TF genelists
#Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
#Qian 11/17/2011 2 channel data use percentile threshold too.
# 6/30/09 Ming-chung. Remove genelist files from /Genelists/AnalysisResults/ folder.
# 6/6/07  Yong consolidated three R files (GetGOComparisonResults.R, GetPathwayComparisonResults.R,
# and GetGeneListComparisonResults.R) into this one unified file: GetGeneSetComparisonResults.R
#
# Modification history from the old GetGeneListComparisonResults.R:
# 5/9/07   Deyun modified for New BroadMIT
# 4/24/07, Yong added Gene Set Global Test using Bioconductor package: globaltest.
# 1/4/07,  Deyun added Gene list file folder
# 12/7/06, Yong modified to replace dget/dput with load/save for R2.4.
# 8/21/06, Yong modified to create MasterAnnotationsTable for multiple gene tables in the output.
# 8/10/06, MC. Include the tables from LS Statistic and Hotelling's T-sq at the top followed
#              by the individual gene tables.
#              Sort genes in the gene table by their p-values in each gene set.
# 7/19/06, Ming-Chung. Add Hotelling T2 test. source("GenesetHotelling.R")
#                            ncomp, alphaHotelling.
# 8/1/05. Ming-Chung
#       Change in the html output "Number of genes:" to
#       "Number of genes used for random variance estimation:", and take out
#       this line entirely when the random variance is not used, in order to
#       avoid confusion with the following line "Number of genes that passed
#       filtering criteria:".
# 8/19/03. Amy modified remove output files from previous analysis.

# The following R variables from VB: alpha
GetGeneSetComparisonResults <- function(cls,
                                        surv = FALSE,
                                        isPaired,
                                        pairID = NULL,
                                        geneId,
                                        filter,
                                        nFilteredGenes,
                                        nGenesForRVM,
                                        doGroupComparison,
                                        grpID,
                                        doqt = FALSE,
                                        corrtest.method,
                                        rvm,
                                        nperm.GSA,
                                        alpha,
                                #        doGlobalTest = F,
                                        isSingleChannel,
                                        pathwayType,
                                        fromKEGGdb,
                                        frommiRTarBasev4,
                                        GeneSetGroup,
                                        organism,
                                        projectPath,
                                        outputName = "GeneSetClassComparison"){

  doGlobalTest <- F
  surv <- ifelse(surv,1,0) # 1: survival; 0: not survival
  ArrayToolsPath <- path.package("GSEA")
  DoGroupComparison <- doGroupComparison
  ProjectPath <- projectPath
  y <- cls
  paired <- IsPaired <- isPaired
  grp <- grpID
  gsanperm <- nperm.GSA
  AnalysisName <- outputName
  nspot <- nFilteredGenes
  PathwayType <- pathwayType
  DoGlobalTest <- doGlobalTest
  GenesTotal <- nGenesForRVM
  IsSingleChannel <- isSingleChannel
  DoRandomVariance <- rvm
  WorkPath <- file.path(projectPath, "Fortran", outputName)
 # GeneSetGroup <- "Pathway"
  if (!dir.exists(WorkPath)) dir.create(WorkPath, recursive = TRUE)
  if (doqt || (surv==1)) {
    DoGroupComparison <- F
    paired <- IsPaired <- F
  }

  if(DoGroupComparison) {
    #Ting 8/11/2014 added to support predicted TF target genelists
    #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
    #Qian 11/17/2011 2 channel data use percentile threshold too.
    # Create for Gene set class comparison extension analysis

    # Load the data and option
    # try(memory.limit(4095))
    load(file.path(WorkPath, "GSEAgroupcomparison.rda"))
    nclasses <- length(unique(y))
    classlabels <- unique(y)
    classsize <- as.integer(table(y))
    ngroups <- length(unique(grp))
    # IsPaired <- FALSE



    codeReg <- FALSE
    surv <- 0

    # Create folders
    ExternalFileName <- paste(ProjectPath, "Output",outputName, sep="/")
    dir.create(ExternalFileName, showWarnings=F, recursive = TRUE)
    ExternalFileName <- paste(ExternalFileName, "/", outputName, ".html", sep="")
 #   dir.create(dirname(ExternalFileName), showWarnings=F)
    unlink(ExternalFileName)

    # Load geneset list
    load(file=paste(WorkPath, "/GeneSetList.rda", sep=""))
    NGeneSets <- length(GeneSetList)
    if (NGeneSets == 0) {
      cat("Stop: The number of total investigated gene sets is zero.\n",
          file=ExternalFileName, append = FALSE)
    } else {
      cat('\n... Number of investigated gene sets is ', NGeneSets, '\n', file=stderr())
      #Create genelist folder.
    #  dir.create(paste(ProjectPath, "/Genelists", sep=""), showWarnings=F)
    #  dir.create(paste(ProjectPath, "/Genelists/AnalysisResults", sep=""), showWarnings=F)

      # create directory for storing significant gene lists
      unlink(paste(ProjectPath, "/Genelists/AnalysisResults/", AnalysisName, sep=""), recursive = TRUE)
      GenesFiltered  <-  nspot

      #added GeneProbesReduced for accomodating gene probe to gene symbole reduction method.
      #modify since the probe reduction is moved to the main dialog, MLI 1/11/2010
      #load(paste(WorkPath, "/ProbeReductionIndex.rda", sep=""))
      #GeneProbesReduced <- length(ProbeReductionIndex)

      KSwarning <- FALSE # always run RVM
      if (nclasses == 2) {
        TestType <- "Two-sample T-test"
      } else {
        TestType <- "F-test"
      }

      if (tolower(GeneSetGroup) == "go") { GeneSetType <- "Gene Ontology" }
      if (tolower(GeneSetGroup) == "pathway") { GeneSetType <- paste(PathwayType, " Pathway") }
      if (tolower(GeneSetGroup) == "genelist") { GeneSetType <- "User defined genelist" }
      if (tolower(GeneSetGroup) == "transcriptionfactor") { GeneSetType <- "Experimentally Verified Transcription Factor Target" } # Ting modified wording 8/11/2014
      if (tolower(GeneSetGroup) == "transcriptionfactorpredict") { GeneSetType <- "Predicted Transcription Factor Target" } # Ting modified to support predicted TF genelist 8/11/2014
      if (tolower(GeneSetGroup) == "microrna") { GeneSetType <- "MicroRNA Target" }
      if (tolower(GeneSetGroup) == "proteindomain") { GeneSetType <- "Protein Domain" }
      if (tolower(GeneSetGroup) == "broad") { GeneSetType <- "Broad Institute - Molecular Signature Database (MSigDB)" }
      if (tolower(GeneSetGroup) == "lymphoid") { GeneSetType <- "Lymphoid signatures - Gene Expression Signatures for Lymphoid Biology" }

      ####run LS test of gene sets
      cat('... Run LS test ...',
          '    Please wait while the program is still running', sep="\n", file=stderr())
   #   source.out<-try(source(paste(ArrayToolsPath,"/R/GenesetTests.R", sep=""), echo = FALSE))
   #   if (class(source.out)=='try-error') { cat(geterrmessage()); stop(); }
      system.time(classifiersLS <- ConductLSgroupcomparison(B=gsanperm, RVM=DoRandomVariance, filepath = WorkPath, FilterAnalysisSet = filter))
      gc()
      save(classifiersLS, file=file.path(WorkPath, "classifiersLS.rda"))
      cat('\n', file=stderr())
      hasLS <- TRUE

      ####run Efron-Tibshirani's GSA method
      cat('... Run Efron-Tibshirani GSA maxmean test ...',
          paste('    Number of permutations is ', gsanperm, sep=""),
          '    Please wait while the program is still running', sep="\n",
          file=stderr())
      system.time(classifiersGSA <- ConductMaxmeangroupcomparison(B=gsanperm, WorkPath))
      gc()
      save(classifiersGSA, file=file.path(WorkPath, "classifiersGSA.rda"))
      if (is.null(classifiersGSA)) cat('... GSA error: no GSA result available.\n', file=stderr())
      cat('\n', file=stderr())
      pHotelling <- NULL

      ####combining results from all tests
      cat("Combining test results for HTML output table ... \n", file=stderr())
      pStat <- data.frame(classifiersLS$res$pValue, classifiersGSA$res$pValue)
      names(pStat) <- c("pLS","pGSA")

      pIndex <- which(rowSums(pStat <= alpha, na.rm=T) > 0)
      sigGeneSet <- length(pIndex)
      cat('Total number of significant gene sets is ', sigGeneSet, '\n', file=stderr())
      if (sigGeneSet > 0) {
        pStat <- pStat[pIndex,,drop=F]
        #generate gene set specific heatmap for all significant gene sets
        cat("Generating heatmaps for significant gene sets ... \n", file=stderr())
        #Qian 11/17/2011 2 channel data use percentile threshold too.
        heatmaptrunclevel <- NULL
        #Prefs <- read.table(paste(ArrayToolsPath, "Prefs/Preferences.txt", sep="/"), header=T, as.is=T, sep="\t", row.names=NULL) #Qian 4/6/09
        #heatmaptrunclevel <- as.numeric(Prefs$Value[Prefs$Variable == "Heatmapthreshold"])
        #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
        CreateGeneSetSpecificHeatmap(geneId = geneId,filter = filter, gsindex=pIndex, gsorder=order(pStat$pLS),
                                     #surv=0,
                                     trunclevel=heatmaptrunclevel,
                                     singlechannel=IsSingleChannel, projectpath=ProjectPath, workpath=WorkPath,
                                     outputpath=paste(ProjectPath, "Output", AnalysisName, "heatmap", sep="/"),
                                     paired = IsPaired, doGroupComparison, grpID)
        cat("Generating the geneset table for significant gene sets ... \n", file=stderr())
        GetGeneSetHTMLOutput <- GetGenesHTML(geneId = geneId, PathwayType = pathwayType,
                                             GeneSetGroup = GeneSetGroup, index=pIndex,
                                             classlabels=classlabels,
                                             paired=IsPaired, DoMasterAnnotations=F, WorkPath=WorkPath,
                                             Hotelling=(!is.null(pHotelling)), GeneSetGlobalTest=FALSE, GeneSetGSA=(!is.null(classifiersGSA)),
                                             pGeneSet=pStat, npermGSA=gsanperm, GSAscore=NULL,
                                             GeneSetUnit=100,
                                             GeneTableCaption=paste("Genes are sorted and grouped within each gene set<BR>", ifelse(!IsPaired, outputClassname(classlabels), "")),
                                             IsSingleChannel = IsSingleChannel,alpha = alpha,
                                             ProjectPath = projectPath, AnalysisName = outputName,fromKEGGdb = fromKEGGdb)
        Master.ind <- unique(GetGeneSetHTMLOutput$GeneIndex)
      }

      ## WRITE OUTPUT:
      UseFrames <- F

      cat("<!-- This is an HTML document.  Please make sure the filename has either"
          , "     an .htm or .html extension, and view this file in a web browser. -->"
          , ""
          , "<HTML>"
          , "<HEAD>"
          , if (surv == 0) "<TITLE>Compare enrichment in two groups of samples</TITLE>"
          , "<STYLE>"
          , "P {font-size: 9pt}"
          , "TABLE {font-size: 9pt}"
          , "</STYLE>"
          , "</HEAD>"
          , "<BODY>"
          , "<SCRIPT>"
          , "function popUp(text) {"
          , "   var prop ="
          , "   \"location=no,scrollbars=yes,menubars=no,toolbars=no,resizable=yes\";"
          , "   popup = window.open(\"\",\"mywin\",prop);"
          , "   popup.document.open();"
          , "   popup.document.write(text);"
          , "   popup.document.close();"
          , "   popup.focus();"
          , "   }"
          , "   </SCRIPT>"
      #    , paste("<A HREF= ",'"file:///'
      #            ,paste(ArrayToolsPath, "/Doc/HTML/GeneSet", ifelse(surv==0, "Class", "Survival"), "ComparisonExt_Results.html", sep=""),'"'
      #            ," TARGET=_blank>HELP </A>",sep="")
          , paste("<P><B><U>Description of the problem:</U></B></P>")
          , "<P>"
          , if (doGroupComparison) paste("\n<BR>Groups:", paste(unique(grpID),collapse = ", "))
          , paste("\n<BR>Classes:",paste(unique(cls),collapse = ", "))
          , paste("\n<BR>Number of classes:", nclasses)
          , if (IsPaired) paste("\n<BR>Samples were paired by: ", paste(unique(pairID),sep = ", "))
          , "<BR>"
          , if (codeReg && !KSwarning) paste("Number of genes used for random variance estimation:",GenesTotal, "<BR>")
          , paste("Number of genes that passed filtering criteria:",GenesFiltered )
          , "<BR>"
          #, paste("Method of handling redundant probes:",ifelse(probereduction==1, "Use all gene probes for a given gene"
          #  , ifelse(probereduction==2, "Use the most variable gene probe as measured by IQR across arrays for a given gene"
          #  , "Use the maximally expressed gene probe as measured by average log intensity across arrays for a given gene")))
          #, "<BR>"
          #, paste("Number of genes used in analysis after handling redundant probes:", GeneProbesReduced)
          #, "<BR>"
          , paste("Type of Gene Sets:", GeneSetType)
          , "<BR>"
          , paste("Number of total investigated Gene Sets:", NGeneSets)
          , "<BR>"
          #, paste("Type of univariate test used: ",TestType)
          , if (!doqt) ifelse(DoRandomVariance, "Random variance model was used<BR>", "Random variance model option: OFF <BR>")
          #, paste(ifelse(codeReg == 0, "<BR>", ifelse(KSwarning, "<BR>Random variance model was not used because distribution assumptions of this model were not satisfied.", "<BR>Random variance model was used.")), sep="")
          # , if (!(tolower(GeneSetGroup) %in% c("go", "pathway"))) paste("Gene Set files can be found in the folder ", "TF", "<BR>")
          # , "<BR>"
      , if (tolower(GeneSetGroup) == "transcriptionfactor") paste("Gene Set files can be found in the folder ", paste0(path.package("GSEA"), "/Transcription_factor/TRED_", tolower(organism)), "<BR>")
      , if (tolower(GeneSetGroup) == "microrna") paste("Gene Set files can be found in the folder ", paste0(path.package("GSEA"), "/MicroRNA/",ifelse(frommiRTarBasev4,"v4/","v7/"),"experimentally_verified_", tolower(organism)), "<BR>")
      , if (tolower(pathwayType) == "kegg") paste("Gene Set files can be found in the folder ", paste0(path.package("GSEA"), "/Pathways/", ifelse(fromKEGGdb, "KEGG","KEGG_new")), "<BR>")
      , "<BR>"
          , paste("Tests used to find significant gene sets are: LS permutation test",
                  ifelse(!is.null(classifiersGSA), ", Efron-Tibshirani's GSA maxmean test", ""), sep="")
          , "<BR>"
          , paste("The threshold of determining significant gene sets is", alpha)
          , "<BR>"
          , paste("Number of permutations for both LS and GSA tests:", gsanperm, "\n<BR>")
          , " ", file=ExternalFileName, sep="\n", append=F)

      cat( "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>", file=ExternalFileName, sep="\n", append=T)
      if (sigGeneSet > 0) {
        cat( "<P><B>Summary of Results:</B>"
             , "<BR><BR>"
             , paste(sigGeneSet, " out of ", NGeneSets, " investigated gene sets passed the ", alpha, " significance threshold")
             , "<BR>"
             , paste("LS permutation test found", sum(pStat$pLS<alpha), "significant gene sets.")
             , "<BR>"
             , ifelse(!is.null(classifiersGSA), paste("Efron-Tibshirani's maxmean test found", sum(pStat$pGSA<=alpha, na.rm=T), "significant gene sets. <BR>"), "")
             , if (tolower(GeneSetGroup) == "go") {
               paste("By Gene Ontologies:",
                     "<BR>&nbsp;&nbsp;&nbsp;", GetGeneSetHTMLOutput$GONumSigCat[1], " out of the", GetGeneSetHTMLOutput$GONumTotCat[1], " investigated Cellular Component (CC) categories", ifelse(GetGeneSetHTMLOutput$GONumSigCat[1]>1, " are ", " is "), " significant.",
                     "<BR>&nbsp;&nbsp;&nbsp;", GetGeneSetHTMLOutput$GONumSigCat[2], " out of the", GetGeneSetHTMLOutput$GONumTotCat[2], " investigated Molecular Function (MF) categories", ifelse(GetGeneSetHTMLOutput$GONumSigCat[2]>1, " are ", " is "), " significant.",
                     "<BR>&nbsp;&nbsp;&nbsp;", GetGeneSetHTMLOutput$GONumSigCat[3], " out of the", GetGeneSetHTMLOutput$GONumTotCat[3], " investigated Biological Process (BP) categories", ifelse(GetGeneSetHTMLOutput$GONumSigCat[3]>1, " are ", " is "), " significant.",
                     "<BR><BR>")
             }
             , "</P>"
             , file=ExternalFileName, append=T)
        #gene set table
        cat( "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
             , "<P><U>Table</U> - Table of Gene Sets: ", sigGeneSet, " gene sets sorted by LS permutation p-value (significant p-values are in red)"
             , "<BR>To access the list of genes within each gene set, click the hyperlinked number of genes for each gene set."
             , "</P>"
             , file=ExternalFileName, sep="\n", append=T)
        if (length(classlabels) == 2) {
          cat("class 1:", as.character(classlabels)[1], " (", classsize[1], " samples), class 2:",
              as.character(classlabels)[2], " (", classsize[2], " samples)\n",
              file=ExternalFileName, sep="", append=T)
        }
        cat( "<TABLE BORDER>"
             , "<CAPTION ALIGN=left></CAPTION>"
             , GetGeneSetHTMLOutput$GeneSetTable1
             , "</TABLE>"
             , file=ExternalFileName, sep="\n", append=T)

        #hyperlink to the set of gene tables
        cat( "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>", file=ExternalFileName, sep="\n", append=T)
        cat( "<P>Links to the genes within gene sets:<BR>"
             , GetGeneSetHTMLOutput$GeneId1
             , "</P>"
             , file=ExternalFileName, sep="\n", append=T)

    #    cat( "<P></P>", "</BODY>", "</HTML>", file=ExternalFileName, sep="\n", append=T)

        # #gene annotation table
        # load(paste(WorkPath, "/GeneIdNames.rda", sep=""))
        # NumGeneIds <- length(GeneIdNames)
        # load(paste(WorkPath, "/GeneId1.rda", sep=""))
        # NumGenesInAnalysis <- nrow(GeneId1)
        # GeneId <- matrix("", nrow=NumGenesInAnalysis, ncol=NumGeneIds)
        # GeneId[,1] <- GeneId1
        # if (NumGeneIds > 1) {
        #   for (i in 2:NumGeneIds) {
        #     load(paste(WorkPath, "/GeneId", i, ".rda", sep=""))
        #     GeneId[,i] <- get(paste("GeneId",i,sep=""))
        #   }
        # }
        #
        # Master.genest <- GeneId[Master.ind,,drop=FALSE]
        # CreateMasterAnnotationsTable(Master.genest, FolderPath=paste(ProjectPath, "/Output/", AnalysisName, sep=""), AnnotationsTableFile="Annotations.html")

      } else {
        cat("<P>No significant gene sets were found by any gene-set test.</P>"
            , file=ExternalFileName, sep="\n", append=T)
     #   cat( "<P></P>", "</BODY>", "</HTML>", file=ExternalFileName, sep="\n", append=T)
      }

    }

    cat( "<HR SIZE=1 WIDTH=\"100%\" NOSHADE>", file=ExternalFileName, sep="\n", append=T)
    cat( paste0("<P align=\"center\"> [package GSEA version ", packageVersion("GSEA"),"] </P>"), file=ExternalFileName, sep="\n", append=T)
    cat( "<P></P>", "</BODY>", "</HTML>", file=ExternalFileName, sep="\n", append=T)

  } else {
    if (!doqt){
      LR.line1 <- scan(paste(WorkPath,"/LR_data.txt", sep=""),nlines=1, quiet = T)
      n <- scan(paste(WorkPath,"/LR_data.txt", sep=""),skip=1,nlines=1, quiet = T)
      ngroups <- LR.line1[1]
      paired <- LR.line1[3] == 1
      nsamp <- sum(n)

      params <- scan(paste(WorkPath,"/class_perm_params.txt", sep=""), quiet = T)
      codeReg  <- params[2] == 1
   #   surv <- params[3]
    } else{
      ngroups <- 0
      paired <- F
      n <- nFilteredGenes
      codeReg <- F
   #   surv <- F
    }


    #Must create each level of path in a separate dir.create() function call.
    ExternalFileName <- paste(ProjectPath, "/Output/", AnalysisName, sep="")
    if (!dir.exists(ExternalFileName))
      dir.create(ExternalFileName, showWarnings=F, recursive = TRUE)
    ExternalFileName <- paste(ExternalFileName, "/", AnalysisName, ".html", sep="")
#    dir.create(dirname(ExternalFileName), showWarnings=F)
    unlink(ExternalFileName)

    GeneSet.line1 <- scan(paste(WorkPath,"/GeneSet_data.txt", sep=""),nlines=1, quiet = T)
    NGeneSets <- GeneSet.line1[1]
    if (NGeneSets == 0) {
      cat("Stop: The number of total investigated gene sets is zero.\n",
          file=ExternalFileName, append = FALSE)
    } else {
      cat('\n... Number of investigated gene sets is ', NGeneSets, '\n', file=stderr())
      #Create genelist folder.
      # dir.create(paste(ProjectPath, "/Genelists", sep=""), showWarnings=F) #Ting 7/18/19
      # dir.create(paste(ProjectPath, "/Genelists/AnalysisResults", sep=""), showWarnings=F) #Ting 7/18/19

      #delete existing same-named genelist file. note GeneListFileName defined in RunR.R.
      #curgenelistfile<-paste(ProjectPath, "/Genelists/AnalysisResults/", GeneListFileName, sep="")
      #unlink(curgenelistfile)
      # create directory for storing significant gene lists
      # unlink(paste(ProjectPath, "/Genelists/AnalysisResults/", AnalysisName, sep=""), recursive = TRUE) #Ting 7/18/19
      #dir.create(paste(ProjectPath, "/Genelists/AnalysisResults/", AnalysisName, sep="")) MLI 6/30/2009

      GenesFiltered  <-  nspot

      #added GeneProbesReduced for accomodating gene probe to gene symbole reduction method.
      #modify since the probe reduction is moved to the main dialog, MLI 1/11/2010
      #load(paste(WorkPath, "/ProbeReductionIndex.rda", sep=""))
      #GeneProbesReduced <- length(ProbeReductionIndex)

      if (!doqt){
        KSwarning <- FALSE
        # Modified Ming-Chung Li 6/8/05. Add verifying if RVM estimation successes
        regparfile <- paste(WorkPath,"/RegPar.txt", sep="")
        if(codeReg && length(readLines(regparfile)) > 1) {
          RegPar <- scan(paste(WorkPath,"/RegPar.txt", sep=""),sep=",",skip=1,nlines=1,quiet=T)
          RegParA <- RegPar[1]
          RegParB <- RegPar[2]
          RegParKS <- RegPar[3]
          KSwarning <- RegParKS > 0.05
        } else if (codeReg) {
          KSwarning <- TRUE
        }

        if(paired) {
          TestType <- "Paired T-test"
        } else {
          if (surv == 0) {
            if (ngroups == 2) {
              TestType <- "Two-sample T-test"
            } else {
              TestType <- "F-test"
            }
          } else {
            TestType <- "Cox proportional hazard model"
          }
        }
      } else { #qt data
        if (corrtest.method == "spearman")
          TestType <- "Spearman's Correlation test for association between gene expression and quantitative trait" else
            if (corrtest.method =="pearson")
              TestType <- "Pearson's Correlation test for association between gene expression and quantitative trait"
      }


      load(paste(WorkPath, "/experiments.rda", sep=""))
      if (surv == 0) {
        classlabels <- unique(sort(experiments$pheno))
        classsize <- as.integer(table(experiments$pheno))
      } else {
        classlabels <- NULL
      }
#
#       if (tolower(GeneSetGroup) == "go") { GeneSetType <- "Gene Ontology" }
       if (tolower(GeneSetGroup) == "pathway") { GeneSetType <- paste(PathwayType, " Pathway") }
#       if (tolower(GeneSetGroup) == "genelist") { GeneSetType <- "User defined genelist" }
       if (tolower(GeneSetGroup) == "transcriptionfactor") { GeneSetType <- "Experimentally Verified Transcription Factor Target" } # Ting modified wording 8/11/2014
#       if (tolower(GeneSetGroup) == "transcriptionfactorpredict") { GeneSetType <- "Predicted Transcription Factor Target" } # Ting modified to support predicted TF genelist 8/11/2014
       if (tolower(GeneSetGroup) == "microrna") { GeneSetType <- "MicroRNA Target" }
#       if (tolower(GeneSetGroup) == "proteindomain") { GeneSetType <- "Protein Domain" }
#       if (tolower(GeneSetGroup) == "broad") { GeneSetType <- "Broad Institute - Molecular Signature Database (MSigDB)" }
#       if (tolower(GeneSetGroup) == "lymphoid") { GeneSetType <- "Lymphoid signatures - Gene Expression Signatures for Lymphoid Biology" } #Qian 3/13/09
 #     GeneSetType <- paste(PathwayType, " Pathway")

      ####get fortran output of LS/KS permutation test of gene sets, one of the default methods
      #Fortran output is already sorted by pLS, we need to re-sort by Index as to be consistent with the output of Hotelling and Global test.
      cat('... Get LS and KS permutation test results ...\n', file=stderr())
      classifiersLS <- read.table(paste(WorkPath, "/DEGs.txt", sep=""), header=F, skip=2)
      names(classifiersLS) <- c("Index","nGenesInGeneSet","statLS","pLS","statKS","pKS")
      resort <- order(classifiersLS$Index)
      classifiersLS <- classifiersLS[resort,,drop=F]
      classifiersLS <- classifiersLS[!is.nan(classifiersLS$statLS),] # Ting 11/12/2019 added the code

      ####run Efron-Tibshirani's GSA method, one of the default methods
      cat('... Run Efron-Tibshirani GSA maxmean test ...',
          paste('    Number of permutations is ', gsanperm, sep=""),
          '    Please wait while the program is still running',
          if (DoGlobalTest && !paired) '    After it is finished, the Goeman global test will continue',
          sep="\n",
          file=stderr())
   #   source.out<-try(source(paste(ArrayToolsPath,"/R/GenesetTests.R", sep=""), echo = FALSE))
   #   if (class(source.out)=='try-error') { cat(geterrmessage()); stop(); }
      classifiersGSA <- ConductGSATest(alpha=alpha, gsaSurv=(surv==1), gsaPaired=paired, gsaQuant = doqt,gsanperm=gsanperm, filepath=WorkPath)
      if (is.null(classifiersGSA)) cat('... GSA function internal error: no GSA result available.\n', file=stderr())

      ####run hotelling's T-square test of gene sets
      hotelling <- F  #1/9/08 Rich instructed to remove hotelling t-square test.
      pHotelling <- NULL
      #if (hotelling && (surv == 0)) {
      #   cat('... Run Hotelling T-square test ...\n', file=stderr())
      #   source.out<-try(source(paste(ArrayToolsPath,"/R/GenesetHotelling.R", sep=""), echo = FALSE))
      #   if (class(source.out)=='try-error') { cat(geterrmessage()); stop(); }
      #   MinSampleSize <- 10
      #   out <- foo(MinSampleSize)
      #   unlink(paste(WorkPath,"/Hotelling.rda",sep=""))
      #   save(out, file = paste(WorkPath,"/Hotelling.rda",sep=""))
      #   if (out$ErrorCode == 0) { pHotelling <- out$pHotelling }
      #}

      ####run global test of gene sets
      goeman <- DoGlobalTest  #1/9/08 run Goeman's global test as one of the default methods
      if (goeman && !paired) {
        cat('... Run Goeman global test ...\n', file=stderr())
     #   source.out<-try(source(paste(ArrayToolsPath,"/R/GenesetTests.R", sep=""), echo = FALSE))
     #   if (class(source.out)=='try-error') { cat(geterrmessage()); stop(); }
        classifiersGT <- ConductGlobalTest(alpha=alpha, gtSurv=(surv==1), filepath=WorkPath)
        hasGT <- T
      } else {
        classifiersGT <- NULL
        hasGT <- F
      }

      ####combining results from all tests
      cat("Combining test results for HTML output table ... \n", file=stderr())
      #if (!is.null(classifiersGSA) && hasGT && !is.null(pHotelling)) {
      #  pStat <- data.frame(classifiersLS$pLS, classifiersLS$pKS, classifiersGSA$pValue, classifiersGT$pValue, pHotelling)
      #  names(pStat) <- c("pLS","pKS","pGSA","pGlobal","pHotelling")
      #}
      if (!is.null(classifiersGSA) && hasGT) {
        pStat <- data.frame(classifiersLS$pLS, classifiersLS$pKS, classifiersGSA$pValue, classifiersGT$pValue)
        names(pStat) <- c("pLS","pKS","pGSA","pGlobal")
      }
      if (!is.null(classifiersGSA) && !hasGT) {
        pStat <- data.frame(classifiersLS$pLS, classifiersLS$pKS, classifiersGSA$pValue)
        names(pStat) <- c("pLS","pKS","pGSA")
      }
      if (is.null(classifiersGSA) && hasGT) {
        pStat <- data.frame(classifiersLS$pLS, classifiersLS$pKS, classifiersGT$pValue)
        names(pStat) <- c("pLS","pKS","pGlobal")
      }
      if (is.null(classifiersGSA) && !hasGT) {
        pStat <- data.frame(classifiersLS$pLS, classifiersLS$pKS)
        names(pStat) <- c("pLS","pKS")
      }

      pIndex <- which(rowSums(pStat < alpha, na.rm=T) > 0)
      sigGeneSet <- length(pIndex)
      cat('Total number of significant gene sets is ', sigGeneSet, '\n', file=stderr())
      if (sigGeneSet > 0) {
        pStat <- pStat[pIndex,,drop=F]
        GSAscore <- classifiersGSA$GSAscore[pIndex] ## Lori 05/06/09
        # MLI 11/10/2008
        if (surv==0) { # MLI 5/4/09 surv == 0
          #generate gene set specific heatmap for all significant gene sets
          cat("Generating heatmaps for significant gene sets ... \n", file=stderr())
          #Qian 11/17/2011 2 channel data use percentile threshold too.
          heatmaptrunclevel <- NULL
          #Qian 2/12/09
          #Prefs <- read.table(paste(ArrayToolsPath, "Prefs/Preferences.txt", sep="/"), header=T, as.is=T, sep="\t", row.names=NULL) #Qian 4/6/09
          #heatmaptrunclevel <- as.numeric(Prefs$Value[Prefs$Variable == "Heatmapthreshold"])
          #EndQian
          #Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
          CreateGeneSetSpecificHeatmap(geneId = geneId, filter = filter, gsindex=pIndex, gsorder=order(pStat$pLS),
                                       #surv=surv,
                                       trunclevel=heatmaptrunclevel,
                                       singlechannel=IsSingleChannel, projectpath=ProjectPath, workpath=WorkPath,
                                       outputpath=paste(ProjectPath, "Output", AnalysisName, "heatmap", sep="/"),
                                       paired, doGroupComparison, grpID)
          #only consider index of LS/KS and/or GSA (excluding global test) for drawing GO graph tree
          #it is found to necessarily use names() to find the correct index.
          #GOTreeIndex <- as.numeric(names(which(rowSums(pStat[,1:3] < alpha, na.rm=T) > 0)))
        }

        if ((surv == 1) || doqt)
          hasClasses <- F else
            hasClasses <- T
        GetGeneSetHTMLOutput <- GetGenesHTML(geneId = geneId, PathwayType = pathwayType,
                                             GeneSetGroup = GeneSetGroup, hasClasses,
                                             index=pIndex, classlabels=classlabels,
                                             paired=IsPaired, DoMasterAnnotations=F, WorkPath=WorkPath,
                                             Hotelling=(!is.null(pHotelling)), GeneSetGlobalTest=hasGT,
                                             GeneSetGSA=(!is.null(classifiersGSA)),
                                             pGeneSet=pStat, npermGSA=gsanperm, GSAscore=GSAscore,
                                             GeneSetUnit=100,
                                             GeneTableCaption=paste("Genes are sorted and grouped within each gene set<BR>", ifelse(!paired && surv==0, outputClassname(classlabels), "")),
                                             IsSingleChannel = IsSingleChannel, alpha = alpha,
                                             ProjectPath = projectPath, AnalysisName = outputName,fromKEGGdb = fromKEGGdb)

        # if (surv==1)
        #   GetGeneSetHTMLOutput <- GetGenesHTML(geneId = geneId, PathwayType = pathwayType, GeneSetGroup = GeneSetGroup,surv = T,
        #                                        index=pIndex, classlabels=classlabels,
        #                                        paired=IsPaired, DoMasterAnnotations=F, WorkPath=WorkPath,
        #                                        Hotelling=(!is.null(pHotelling)), GeneSetGlobalTest=hasGT,
        #                                        GeneSetGSA=(!is.null(classifiersGSA)),
        #                                        pGeneSet=pStat, npermGSA=gsanperm, GSAscore=GSAscore,
        #                                        GeneSetUnit=100,
        #                                        GeneTableCaption=paste("Genes are sorted and grouped within each gene set<BR>", ifelse(!paired && surv==0, outputClassname(classlabels), "")),
        #                                        IsSingleChannel = IsSingleChannel, alpha = alpha,
        #                                        ProjectPath = projectPath, AnalysisName = outputName) else
        # GetGeneSetHTMLOutput <- GetGenesHTML(geneId = geneId, PathwayType = pathwayType, GeneSetGroup = GeneSetGroup,surv = F,
        #                                      index=pIndex, classlabels=classlabels,
        #                                      paired=IsPaired, DoMasterAnnotations=F, WorkPath=WorkPath,
        #                                      Hotelling=(!is.null(pHotelling)), GeneSetGlobalTest=hasGT,
        #                                      GeneSetGSA=(!is.null(classifiersGSA)),
        #                                      pGeneSet=pStat, npermGSA=gsanperm, GSAscore=GSAscore,
        #                                      GeneSetUnit=100,
        #                                      GeneTableCaption=paste("Genes are sorted and grouped within each gene set<BR>", ifelse(!paired && surv==0, outputClassname(classlabels), "")),
        #                                      IsSingleChannel = IsSingleChannel, alpha = alpha,
        #                                      ProjectPath = projectPath, AnalysisName = outputName)


        Master.ind <- unique(GetGeneSetHTMLOutput$GeneIndex)
        #check if GO tree graph is created
        #    if (tolower(GeneSetGroup) == "go") {
        #      GOBPGraphsmall <- paste(ProjectPath, "Output", AnalysisName, "GOBPsmall.png", sep="/")
        #      GOBPGraphlarge <- paste(ProjectPath, "Output", AnalysisName, "GOBPlarge.png", sep="/")
        #      if (file.exists(GOBPGraphsmall)) GOBPTree <- T else GOBPTree <- F
        #      GOMFGraphsmall <- paste(ProjectPath, "Output", AnalysisName, "GOMFsmall.png", sep="/")
        #      GOMFGraphlarge <- paste(ProjectPath, "Output", AnalysisName, "GOMFlarge.png", sep="/")
        #      if (file.exists(GOMFGraphsmall)) GOMFTree <- T else GOMFTree <- F
        #      GOCCGraphsmall <- paste(ProjectPath, "Output", AnalysisName, "GOCCsmall.png", sep="/")
        #      GOCCGraphlarge <- paste(ProjectPath, "Output", AnalysisName, "GOCClarge.png", sep="/")
        #      if (file.exists(GOCCGraphsmall)) GOCCTree <- T else GOCCTree <- F
        #    } else {
        #      GOBPTree <- GOMFTree <- GOCCTree <- F
        #    }
      }

      ## WRITE OUTPUT:
      UseFrames <- F

      cat("<!-- This is an HTML document.  Please make sure the filename has either"
          , "     an .htm or .html extension, and view this file in a web browser. -->"
          , ""
          , "<HTML>"
          , "<HEAD>"
          , if (surv == 0) "<TITLE>Gene Set Class Comparison Results</TITLE>"
          else "<TITLE>Gene Set Surival Analysis Results</TITLE>"
          , "<STYLE>"
          , "P {font-size: 9pt}"
          , "TABLE {font-size: 9pt}"
          , "</STYLE>"
          , paste("<script src='file:///", ArrayToolsPath, "/misc/sorttable.js'></script>", sep="") #Qian 5/29/09 sortable
          , "</HEAD>"
          , "<BODY>"
          , "<SCRIPT>"
          , "function popUp(text) {"
          , "   var prop ="
          , "   \"location=no,scrollbars=yes,menubars=no,toolbars=no,resizable=yes\";"
          , "   popup = window.open(\"\",\"mywin\",prop);"
          , "   popup.document.open();"
          , "   popup.document.write(text);"
          , "   popup.document.close();"
          , "   popup.focus();"
          , "   }"
          , "   </SCRIPT>"
        #  , paste("<A HREF= ",'"file:///'
        #          ,paste(ArrayToolsPath, "/Doc/HTML/GeneSet", ifelse(surv==0, "Class", "Survival"), "Comparison_Results.html", sep=""),'"'
        #          ," TARGET=_blank>HELP </A>",sep="")
          , paste("<P><B><U>Description of the problem:</U></B></P>")
          , "<P>"
          , if ((surv == 0) && (!doqt)) { paste("Number of classes:", ngroups, "\n<BR>Classes:",paste(unique(cls),collapse=", "))
          } #else {
          # paste("Column of the Experiment Descriptors sheet that defines Time variable:",TimeVariable,"\n<BR>",
          #       "Column of the Experiment Descriptors sheet that defines Status variable:",StatusVariable)
          # }
          , "<BR>"
          , if (codeReg && !KSwarning) paste("Number of genes used for random variance estimation:",GenesTotal, "<BR>")
          , paste("Number of genes that passed filtering criteria:",GenesFiltered )
          , "<BR>"
          #, paste("Method of handling redundant probes:",ifelse(probereduction==1, "Use all gene probes for a given gene"
          #  , ifelse(probereduction==2, "Use the most variable gene probe as measured by IQR across arrays for a given gene"
          #  , "Use the maximally expressed gene probe as measured by average log intensity across arrays for a given gene")))
          #, "<BR>"
          #, paste("Number of genes used in analysis after handling redundant probes:", GeneProbesReduced)
          #, "<BR>"
          , paste("Type of Gene Sets:", GeneSetType)
          , "<BR>"
          , paste("Number of total investigated Gene Sets:", NGeneSets)
          , "<BR>"
          , paste("Type of univariate test used: ",TestType)
          , if (!codeReg && !doqt) "<BR>Random variance model for univariate tests: OFF"
          , if(!doqt && codeReg != 0) ifelse(KSwarning, "<BR>Random variance model was not used because distribution assumptions of this model were not satisfied.", "<BR>Random variance model was used.")
          , "<BR>"
   #       , if (!(tolower(GeneSetGroup) %in% c("go", "pathway"))) paste("Gene Set files can be found in the folder ", "DataFilePath", "<BR>")
   , if (tolower(GeneSetGroup) == "transcriptionfactor") paste("Gene Set files can be found in the folder ", paste0(path.package("GSEA"), "/Transcription_factor/TRED_", tolower(organism)), "<BR>")
  #        , "<BR>"
   , if (tolower(GeneSetGroup) == "microrna") paste("Gene Set files can be found in the folder ", paste0(path.package("GSEA"), "/MicroRNA/",ifelse(frommiRTarBasev4,"v4/","v7/"),"experimentally_verified_", tolower(organism)), "<BR>")
   , if (tolower(pathwayType) == "kegg") paste("Gene Set files can be found in the folder ", paste0(path.package("GSEA"), "/Pathways/", ifelse(fromKEGGdb, "KEGG","KEGG_new")), "<BR>")
#   , "<BR>"
          , paste("Tests used to find significant gene sets are: LS/KS permutation test",
                  ifelse(!is.null(classifiersGSA), ", Efron-Tibshirani's GSA maxmean test", ""),
                  ifelse(hotelling && surv == 0, ", Hotelling's T-square test", ""),
                  ifelse(goeman, ", Goeman's global test", ""), sep="")
          , "<BR>"
          , paste("The threshold of determining significant gene sets is", alpha)
          , "<BR>"
          , paste("LS/KS permutation test finds gene sets which have more genes differentially expressed ", ifelse(surv==0, "among the phenotype classes", "with survival times"), " than expected by chance.")
          , "<BR>"
          , ifelse(!is.null(classifiersGSA), paste("Efron-Tibshirani's test uses 'maxmean' statistics to identify gene sets differentially expressed.<BR>"), "Efron-Tibshirani's GSA test could not be run because the requirement of 'GSA' R package was not met.<BR>")
          , ifelse(hotelling && surv==0, paste("Hotelling's T-square test finds gene sets against the null hypothesis that no genes in the set are differentially expressed among the phenotype classes.<BR>"), "")
          , ifelse(goeman, paste("Goeman's Global test finds gene sets which are associated ", ifelse(surv==0, "with the phenotype classes.", "with survival times."), "<BR>"), "")
          , " ", file=ExternalFileName, sep="\n", append=F)

      if (hotelling && surv == 0) {
        cat(paste("The number of principle components used for Hotelling's T-square test is ", ncomp, "<BR>"), file=ExternalFileName, sep="\n", append=T)
        switch(out$ErrorCode
               , cat(paste("The number of total arrays is less than the minimum requirement (", MinSampleSize,
                           "). The Hotelling's T-square test will not be performed.<br>\n", sep = "")
                     , file=ExternalFileName, append=T)
               , cat("Some genes have missing values in ALL arrays. The Hotelling T-square test will not be performed.<br>\n"
                     , file=ExternalFileName, append=T)
               , cat("The Hotelling's T-square test is not able to be performed with your design.<br>\n"
                     , file=ExternalFileName, append=T))
      }

      cat( "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>", file=ExternalFileName, sep="\n", append=T)
      if (sigGeneSet > 0) {
        cat( "<P><B>Summary of Results:</B>"
             , "<BR><BR>"
             , paste(sigGeneSet, " out of ", NGeneSets, " investigated gene sets passed the ", alpha, " significance threshold", ifelse(hotelling || goeman, "among at least one test", ""))
             , "<BR>"
             , paste("LS/KS permutation test found", sum(pStat$pLS<alpha | pStat$pKS<alpha, na.rm=T), "significant gene sets.")
             , "<BR>"
             , ifelse(!is.null(classifiersGSA), paste("Efron-Tibshirani's maxmean test found", sum(pStat$pGSA<=alpha, na.rm=T), "significant gene sets (under", gsanperm, "permutations). <BR>"), "") # GSA is permutation based. We modify to use "<=" instead of "<". MLI 1/14/09
             , ifelse(hotelling && surv==0 && out$ErrorCode==0, paste("Hotelling's T-square test found", sum(pStat$pHotelling<alpha, na.rm=T), "significant gene sets. <BR>"), "")
             , ifelse(goeman, ifelse(hasGT, paste("Goeman's Global test found", sum(pStat$pGlobal<alpha, na.rm=T), "significant gene sets.<BR>"), "Goeman's global test is not available for paired data.<BR>"), "")
             , if (tolower(GeneSetGroup) == "go") {
               paste("By Gene Ontologies:",
                     "<BR>&nbsp;&nbsp;&nbsp;", GetGeneSetHTMLOutput$GONumSigCat[1], " out of the", GetGeneSetHTMLOutput$GONumTotCat[1], " investigated Cellular Component (CC) categories", ifelse(GetGeneSetHTMLOutput$GONumSigCat[1]>1, " are ", " is "), " significant.",
                     "<BR>&nbsp;&nbsp;&nbsp;", GetGeneSetHTMLOutput$GONumSigCat[2], " out of the", GetGeneSetHTMLOutput$GONumTotCat[2], " investigated Molecular Function (MF) categories", ifelse(GetGeneSetHTMLOutput$GONumSigCat[2]>1, " are ", " is "), " significant.",
                     "<BR>&nbsp;&nbsp;&nbsp;", GetGeneSetHTMLOutput$GONumSigCat[3], " out of the", GetGeneSetHTMLOutput$GONumTotCat[3], " investigated Biological Process (BP) categories", ifelse(GetGeneSetHTMLOutput$GONumSigCat[3]>1, " are ", " is "), " significant.",
                     "<BR><BR>")#,
               #               "The Gene Ontology tree graphs are available respectively for Biological Process, Molecular Function, and Cellular Component if in each category there is at least one significant GO category found by either LS/KS or GSA method.",
               #               "The GO tree graph highlights all the significant GO categories from LS/KS and/or GSA methods as well as their parent and ancestor GO categories up to the root category.<BR>",
               #           ifelse(!GOBPTree && !GOMFTree && !GOCCTree, paste("The Gene Ontology tree graphs are not available since there is no significant GO category found by either LS/KS or GSA method.<BR>"), "Click to view the enlarged graph.<BR>"),
               #               ifelse(GOBPTree, paste("<A HREF=\"file:///", GOBPGraphlarge, "\" TARGET=_blank>", "<IMG SRC=\"", GOBPGraphsmall, "\">", "</A>", sep=""), ""),
               #               ifelse(GOMFTree, paste("<A HREF=\"file:///", GOMFGraphlarge, "\" TARGET=_blank>", "<IMG SRC=\"", GOMFGraphsmall, "\">", "</A>", sep=""), ""),
               #               ifelse(GOCCTree, paste("<A HREF=\"file:///", GOCCGraphlarge, "\" TARGET=_blank>", "<IMG SRC=\"", GOCCGraphsmall, "\">", "</A>", sep=""), ""))
             }
             , "</P>"
             , file=ExternalFileName, append=T)
        #gene set table
        cat( "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
             , "<P><U>Table</U> - Table of Gene Sets: ", sigGeneSet, " gene sets sorted by LS permutation p-value (significant p-values are in red)"
             , "<BR>To access the list of genes within each gene set, click the hyperlinked number of genes for each gene set."
             , "</P>"
             , file=ExternalFileName, sep="\n", append=T)
        if (length(classlabels) == 2) {
          cat("class 1:", as.character(classlabels)[1], " (", classsize[1], " samples), class 2:",
              as.character(classlabels)[2], " (", classsize[2], " samples)\n",
              file=ExternalFileName, sep="", append=T)
        }
        #rsm 122413: add file:/// in line below for firefox browser compatibility
        #rsm 122413: suppress output if KEGG not requested
        if (exists("PathwayType") && PathwayType == "Kegg"){
          cat(paste('<P>Color Coded KEGG Pathway Graphs <A HREF=\"file:///',ArrayToolsPath,'/misc/ColorCodeKEGG.html','"','>Legend</A></P>',sep=''), file=ExternalFileName, sep="\n", append=T)
        }
        #end rsm 122413
        cat( GetGeneSetHTMLOutput$GeneSetTable1 #Qian 5/29/09
             , file=ExternalFileName, sep="\n", append=T)

        #hyperlink to the set of gene tables
        cat( "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>", file=ExternalFileName, sep="\n", append=T)
        cat( "<P>Links to the genes within gene sets:<BR>"
             , GetGeneSetHTMLOutput$GeneId1
             , "</P>"
             , file=ExternalFileName, sep="\n", append=T)

        #cat( "<P></P>", "</BODY>", "</HTML>", file=ExternalFileName, sep="\n", append=T) #Qian 5/29/09

        #gene annotation table
        # load(paste(WorkPath, "/GeneIdNames.rda", sep=""))
        # NumGeneIds <- length(GeneIdNames)
        # load(paste(WorkPath, "/GeneId1.rda", sep=""))
        # NumGenesInAnalysis <- nrow(GeneId1)
        # GeneId <- matrix("", nrow=NumGenesInAnalysis, ncol=NumGeneIds)
        # GeneId[,1] <- GeneId1
        # if (NumGeneIds > 1) {
        #   for (i in 2:NumGeneIds) {
        #     load(paste(WorkPath, "/GeneId", i, ".rda", sep=""))
        #     GeneId[,i] <- get(paste("GeneId",i,sep=""))
        #   }
        # }

        # GeneId <- as.matrix(geneId)
        # GeneIdNames <- colnames(geneId)
        # NumGenesInAnalysis <- nrow(geneId)
        # NumGeneIds <- ncol(geneId)
        # for (i in 1:NumGeneIds) {
        #   GeneId[,i] <- gsub("@#$%&", "'", geneId[,i], fixed=T)
        #   GeneId[,i] <- gsub("@#$%!", "\"", geneId[,i], fixed=T)
        # }
        #
        #
        # Master.genest <- GeneId[Master.ind,,drop=FALSE]
        # CreateMasterAnnotationsTable(Master.genest, FolderPath=paste(ProjectPath, "/Output/", AnalysisName, sep=""), AnnotationsTableFile="Annotations.html")

      } else {
        cat("<P>No significant gene sets were found by any gene-set test.</P>"
            , file=ExternalFileName, sep="\n", append=T)
     #   cat( "<P></P>", "</BODY>", "</HTML>", file=ExternalFileName, sep="\n", append=T)
      }

    }
    cat( "<HR SIZE=1 WIDTH=\"100%\" NOSHADE>", file=ExternalFileName, sep="\n", append=T)
    cat( paste0("<P align=\"center\"> [package GSEA version ", packageVersion("GSEA"),"] </P>"), file=ExternalFileName, sep="\n", append=T)
    cat( "<P></P>", "</BODY>", "</HTML>", file=ExternalFileName, sep="\n", append=T)

  }

  return(GetGeneSetHTMLOutput$GeneSetTable)


}
