## ----setup, echo=FALSE, results="hide"-----------------------------------
knitr::opts_chunk$set(tidy=FALSE, cache=TRUE,
                      dev="png",
                      message=FALSE, error=FALSE, warning=FALSE)

## ----install_depend, eval = FALSE----------------------------------------
#  if (!requireNamespace("BiocManager", quietly = TRUE))
#      install.packages("BiocManager")
#  BiocManager::install("Biobase")
#  install.packages(c("GSA","bitops","Cairo"), repos = "https://cran.rstudio.com")

## ----install_GSEA_source, eval = FALSE-----------------------------------
#  if(!requireNamespace("remotes", quietly = TRUE))
#    install.packages("remotes", repos = "https://cran.rstudio.com")
#  remotes::install_local("GSEA_0.1.0.tar.gz", dependencies = TRUE)

## ----pathway, eval = FALSE-----------------------------------------------
#  # KEGG pathways from the KEGG.db R package
#  list.files(system.file("Pathways/KEGG", package = "GSEA"))
#  # KEGG pathways from the KEGGREST R package
#  list.files(system.file("Pathways/KEGG_new", package = "GSEA"))
#  # Biocarat pathways
#  list.files(system.file("Pathways/Biocarta", package = "GSEA"))

## ----keggdesc------------------------------------------------------------
desc <- read.table(system.file("Pathways/Names_KEGG.txt", package = "GSEA"), sep="\t", header=F)
if(!requireNamespace("DT", quietly = TRUE) )
  install.packages("DT", repos = "https://cran.rstudio.com")
DT::datatable(head(desc, n = nrow(desc)), options = list(pageLength = 5)) 


## ----keggdesc2-----------------------------------------------------------
desc <- read.table(system.file("Pathways/Names_KEGG_new.txt", package = "GSEA"), sep="\t", header=F)
DT::datatable(head(desc, n = nrow(desc)), options = list(pageLength = 5)) 


## ----biodesc-------------------------------------------------------------
desc <- read.table(system.file("Pathways/Names_Biocarta.txt", package = "GSEA"), sep="\t", header=F)
DT::datatable(head(desc, n = nrow(desc)), options = list(pageLength = 5))

## ----tf, eval = FALSE----------------------------------------------------
#  # Gene sets of transcription factor targets for human
#  list.files(system.file("Transcription_factor/Tred_human", package = "GSEA"))

## ----mirna, eval = FALSE-------------------------------------------------
#  # Gene sets of microRNA targets for human from miRTarBase v4
#  list.files(system.file("MicroRNA/v4/experimentally_verified_human", package = "GSEA"))
#  # Gene sets of microRNA targets for human from miRTarBase v7
#  list.files(system.file("MicroRNA/v7/experimentally_verified_human", package = "GSEA"))

## ----classexp------------------------------------------------------------
library(GSEA)
dataset<-"Brca"
# gene IDs
geneId <- read.delim(system.file("extdata", paste0(dataset, "_GENEID.txt"),
                                 package = "GSEA"), as.is = TRUE, colClasses = "character")
# gene expression
x <- read.delim(system.file("extdata", paste0(dataset, "_LOGRAT.TXT"),
                            package = "GSEA"), header = FALSE)
filter <- scan(system.file("extdata", paste0(dataset, "_FILTER.TXT"),
                           package = "GSEA"), quiet = TRUE)
# sample information
expdesign <- read.delim(system.file("extdata", paste0(dataset, "_EXPDESIGN.txt"),
                                    package = "GSEA"), as.is = TRUE)
ind1 <- which(expdesign[, 4] == "BRCA1")
ind2 <- which(expdesign[, 4] == "BRCA2")
ind <- c(ind1, ind2)
expr <- x[, ind]
colnames(expr) <- expdesign[ind, 1]
projectPath <- file.path(Sys.getenv("HOME"),"Brca")
outputName <- "GeneSetClassComparison"
cls <- c(rep("BRCA1", length(ind1)), rep("BRCA2", length(ind2)))
res.class <- gsea(expr = expr,
            filter = filter,
            geneId = geneId,
            cls = cls,
            geneSetType = "BioCarta",
            isSingleChannel = FALSE,
            alpha = 0.005,
            organism = "human",
            projectPath = projectPath,
            outputName = outputName,
            popHTML = FALSE)
DT::datatable(res.class, options = list(pageLength = 5))

## ----survexp-------------------------------------------------------------

dataset<-"Pomeroy"
# gene IDs
geneId <- read.delim(system.file("extdata", paste0(dataset, "_GENEID.txt"),
                                 package = "GSEA"), as.is = TRUE, colClasses = "character")
# expression data
x <- read.delim(system.file("extdata", paste0(dataset, "_LOGINT.TXT"),
                            package = "GSEA"), header = FALSE)
# filter information, 1 - pass the filter, 0 - filtered
filter <- scan(system.file("extdata", paste0(dataset, "_FILTER.TXT"),
                           package = "GSEA"), quiet = TRUE)
# sample information
expdesign <- read.delim(system.file("extdata", paste0(dataset, "_EXPDESIGN.txt"),
                                   package = "GSEA"), as.is = TRUE)
time <- expdesign[,7]
status <- expdesign[,12]
ind1 <- which(status == 0)
ind2 <- which(status == 1)
ind <- c(ind1, ind2)
expr <- x[, ind]
time <- time[ind]
status <- status[ind]
colnames(expr) <- expdesign[ind, 1]
projectPath <- file.path(Sys.getenv("HOME"),dataset)
outputName <- "GeneSetSurvivalComparison"
res.surv <- gsea(expr = expr,
                 filter = filter,
                 surv = T,
                 time = time,
                 status = status,
                 geneId = geneId,
                 rvm = FALSE,
                 geneSetType = "BioCarta",
                 alpha = 0.005,
                 organism = "human",
                 projectPath = projectPath,
                 outputName = outputName,
                 popHTML = FALSE)
DT::datatable(res.surv, options = list(pageLength = 5))

## ----qtexp---------------------------------------------------------------

dataset<-"Brca"
# gene IDs
geneId <- read.delim(system.file("extdata", paste0(dataset, "_GENEID.txt"),
                                 package = "GSEA"), as.is = TRUE, colClasses = "character")
# expression data
expr <- read.delim(system.file("extdata", paste0(dataset, "_LOGRAT.TXT"),
                               package = "GSEA"), header = FALSE)
# filter information, 1 - pass the filter, 0 - filtered
filter <- scan(system.file("extdata", paste0(dataset, "_FILTER.TXT"),
                           package = "GSEA"), quiet = TRUE)
# sample information
expdesign <- read.delim(system.file("extdata", paste0(dataset, "_EXPDESIGN.txt"),
                                    package = "GSEA"), as.is = TRUE)
quant <- expdesign[,11]
colnames(expr) <- expdesign[, 1]
projectPath <- file.path(Sys.getenv("HOME"),"Brca")
outputName <- "GeneSetQTComparison"
res.qt <- gsea(expr = expr,
               filter = filter,
               quant = quant,
               geneId = geneId,
               geneSetType = "KEGG",
               fromKEGGdb = FALSE,
               isSingleChannel = FALSE,
               alpha = 0.005,
               organism = "human",
               corrtest.method = "pearson",
               projectPath = projectPath,
               outputName = outputName,
               popHTML = FALSE)
DT::datatable(res.qt, options = list(pageLength = 5))

## ----sessionInfo---------------------------------------------------------
sessionInfo()

